# ===================================================================
# The following R code requires the use of an independent R package named
# "lsei" that is downloadable at the web site:
# 
#     http://www.stat.auckland.ac.nz/~yongwang/
# ===================================================================

library(lsei)


# ==========
# Estimation
# ==========

initial = function(x, beta, m, mix, start) UseMethod("initial")
valid = function(x, beta, m, mix) UseMethod("valid")
logd = function(x, beta, m, pt, which) UseMethod("logd")

cnmms = function(x, init=NULL, start=1, which=c(1,1,0,0), 
                 model=c("normmix", "spsym"),
                 grid=100, tol=1e-10, maxit=1000, llt=NULL) {
  model = match.arg(model)
  class(x) = model
  k = length(x)
  if(is.null(c(init$mix,init$m))) init = initial.snpmle(x, init, start=start)
  beta = init$beta
  m = init$m
  mix = init$mix
  ll1 = -Inf
  convergence = 1
  gridpoints = seq(min(x), max(x), length=grid)
  for(i in 1:maxit) {
    l = logd(x, beta, m, mix$pt, which=c(1,rep(0,4)))$ld
    ma = apply(l, 1, max)
    dmix = drop(exp(l - ma) %*% mix$pr) + 1e-100
    g = maxgrad(x, beta, m, dmix, ma, grid=gridpoints, tol=-Inf)
    gradient = max(g$grad)
    mix = dden(c(mix$pt,g$pt), c(mix$pr,rep(0,length(g$pt))))
    lpt = logd(x, beta, m, mix$pt, which=c(1,rep(0,4)))$ld
    dpt = pmin(exp(lpt - ma), 1e100)
    a = cbind(dpt/dmix - drop(rep(2,k)))
    r = nnls(rbind(a, rep(1,length(mix$pt))), c(rep(0,nrow(a)),1))
    sol = r$x / sum(r$x)
    r = lsch(mix, beta, m, dden(mix$pt,sol), beta, m, x,
                 which=c(1,rep(0,3)), 
		 model=model)
    mix = collapse.snpmle(r$mix, beta, m, x)
    r = bfgs(mix, beta, m, x, which=which, model=model)
    if(model=="spsym") {
      r$mix = unique(dden(c(r$mix$pt,2*r$m-r$mix$pt),
                 c(r$mix$pr,r$mix$pr)), 
		 prec=1e-5)
      nummixpt = length(r$mix$pt)
      if(nummixpt != 1) {
        if(nummixpt%%2 == 1) {
          which[4] = 0
          index = (nummixpt + 1)/2
          r$mix = dden(c(r$mix$pt[1:index]), c(r$mix$pr[1:(index-1)], 
					     r$mix$pr[index]/2))
        }
        else {
          index = nummixpt/2
          r$mix = dden(c(r$mix$pt[1:index]), c(r$mix$pr[1:index]))
        }
      } else which[4] = 0
      k1 = if(which[1]) length(r$mix$pr) - 1 else 0
      dl = dll.snpmle(x, r$mix, r$beta, r$m, which=c(1,which))
      r$grad = c(if(which[1]) dl$dp[-(k1+1)] - dl$dp[k1+1] else NULL,
                 if(which[2]) dl$dt else NULL,
                 if(which[3]) dl$db else NULL,
                 if(which[4]) dl$dm else NULL)
    }
    beta = r$beta
    m = r$m
    mix = r$mix
    if(r$conv == 3) {
      l = logd(x, beta, m, mix$pt, which=c(1,rep(0,4)))$ld
      ma = apply(l, 1, max)
      dmix = drop(exp(l - ma) %*% mix$pr) + 1e-100
      grad = grad(x, mix$pt, beta, m, dmix, ma, order=0)$d0
      gradient = max(grad)
      if(length(gradient) == 1 && isTRUE(all.equal(gradient,0))) r$conv = 0
      convergence = r$conv
      break
    }
    if(is.null(llt)){if(r$ll >= ll1 && r$ll <= ll1 + tol){convergence=0; break}}
    else {if(r$ll >= llt) {convergence = 0; break}
          else if(r$ll >= ll1 && r$ll <= ll1 + 1e-16) {convergence=1; break}}
    ll1 = r$ll
  }
  mixall = NULL
  if(model=="spsym") mixall = sort(unique(dden(c(mix$pt,2*m-mix$pt),c(mix$pr,mix$pr)),prec=1e-6))
  if(model=="normmix") {
    list(mix=mix, beta=beta, num.iterations=i, ll=r$ll, grad=r$grad,
         max.gradient=gradient, convergence=convergence)
  }
  else {
    list(mixall=mixall, mix=mix, beta=beta, m=m, num.iterations=i,
         ll=r$ll, grad=r$grad, max.gradient=gradient, convergence=convergence)
  }
}

maxgrad = function(x, beta, m, dmix, ma, grid=100, tol=-Inf, maxit=100) {
  if( length(grid) == 1 ) grid = seq(min(x), max(x), length=grid)
  np = length(grid)
  dg = grad(x, grid, beta, m, dmix, ma, order=1)$d1
  jmax = (1:(np-1))[dg[1:(np-1)] > 0 & dg[2:np] < 0]
  if( length(jmax) < 1 ) return
  pt = (grid[jmax] + grid[jmax+1]) * .5
  left = grid[jmax]
  right = grid[jmax+1]
  if(length(pt) != 0) {
    for( i in 1:maxit ) {
      pt.old = pt
      d = grad(x, pt, beta, m, dmix, ma, order=1:2)
      d1 = d$d1
      d2 = d$d2
      left[d1>0] = pt[d1>0]
      right[d1<0] = pt[d1<0]
      pt = pt - d1 / d2
      j = is.na(pt) | pt < left | pt > right
      pt[j] = (left[j] + right[j]) * .5
      if( max(abs(pt - pt.old)) <= 1e-14 * diff(range(grid))) break
    }
  }
  else i = 0
  if(dg[np] >= 0) pt = c(pt, grid[np])
  if(dg[1] <= 0) pt = c(grid[1], pt)
  if(length(pt) == 0) stop("no new support point found")
  g = grad(x, pt, beta, m, dmix, ma, order=0)$d0
  names(pt) = names(g) = NULL
  j = g >= tol
  list(pt=pt[j], grad=g[j], num.iterations=i)
}

grad = function(x, pt, beta, m, dmix, ma, order=0) {
  if(is.dden(dmix)) {
    l = logd(x, beta, m, dmix$pt, which=c(1,rep(0,4)))$ld
    ma = apply(l, 1, max)
    dmix = drop(exp(l - ma) %*% dmix$pr) + 1e-100
  }
  g = vector("list", length(order))
  names(g) = paste("d", order, sep="")
  which = c(1,rep(0,4))
  if(any(order >= 1)) which[3:max(order+2)] = 1
  dl = logd(x, beta, m, pt, which=which)
  dpt = pmin(exp(dl$ld - ma), 1e100)
  if(any(0 == order)) g$d0 = colSums(dpt / dmix) - length(x)
  if(any(1 == order)) g$d1 = colSums(dpt * dl$dt1 / dmix)
  if(any(2 == order)) g$d2 = colSums(dpt * (dl$dt1 * dl$dt1 + dl$dt2) / dmix)
  g
}

bfgs = function(mix, beta, m, x, tol=1e-16, maxit=100, which=c(1,1,0,0),
                model=c("normmix", "spsym"), D=NULL) {
  model = match.arg(model)
  k1 = if(which[1]) length(mix$pr) - 1 else 0
  k2 = if(which[2]) length(mix$pt) else 0
  k3 = if(which[3]) length(beta) else 0
  k4 = if(which[4]) length(m) else 0
  if( k1 == 0 ) which[1] = 0
  if( sum(which) == 0 ) stop("No parameter specified to be updated in bfgs()")
  if( is.null(D) ) D = diag(-1, nrow=k1+k2+k3+k4)
  else if( nrow(D) != k1+k2+k3+k4 ) stop("D has unmatching dimensions")
  dl = dll.snpmle(x, mix, beta, m, which=c(1,which))
  ll = dl$ll
  grad = c(if(which[1]) dl$dp[-(k1+1)] - dl$dp[k1+1] else NULL,
           if(which[2]) dl$dt else NULL,
           if(which[3]) dl$db else NULL,
           if(which[4]) dl$dm else NULL)
  r = list(mix=mix, beta=beta, m=m, ll=ll, grad=grad, convergence=1)
  prmt = c(if(which[1]) r$mix$pr[-(k1+1)] else NULL,
           if(which[2]) r$mix$pt else NULL,
           if(which[3]) r$beta else NULL,
           if(which[4]) r$m else NULL)
  for(i in 1:maxit) {
    old.r = r
    old.prmt = prmt
    prmt2 = drop( prmt - D %*% r$grad )
    d1 = prmt2 - prmt
    alpha1 = alpha2 = alpha3 = 1
    if(which[1]) {
      pr = prmt2[1:k1]
      pr2 = c(pr, 1-sum(pr))
      if(any(pr2 < 0)) {
        step = pr2 - r$mix$pr
        ratio = pmax(-pr2, 0) / abs(step)
        jmax = which.max(ratio)
        alpha1 = 1 - ratio[jmax]
      }
    }
    else pr2 = r$mix$pr
    if(which[3]) beta2 = prmt2[(k1+k2+1):(k1+k2+k3)] else beta2 = r$beta
    if(which[2] && which[4]) {
      pt2 = prmt2[(k1+1):(k1+k2)]
      m2 = prmt2[(k1+k2+k3+1):(k1+k2+k3+k4)]
    }
    else {
      pt2 = r$mix$pt
      m2 = r$m
    }
    alphas = c(alpha1, alpha2, alpha3)
    jmin = which.min(alphas)
    alpha = alphas[jmin]
    prmt2 = prmt + alpha * d1
    if(which[1]) {
      pr = prmt2[1:k1]
      pr2 = c(pr, 1-sum(pr))
      if (alpha < 1 && jmin == 1) pr2[jmax] = 0
    }
    else pr2 = r$mix$pr
    if(which[2]) pt2 = prmt2[(k1+1):(k1+k2)] else pt2 = r$mix$pt
    mix2 = dden(pt2, pr2)
    if(which[3]) beta2 = prmt2[(k1+k2+1):(k1+k2+k3)] else beta2 = r$beta
    if(which[4]) m2 = prmt2[(k1+k2+k3+1):(k1+k2+k3+k4)] else m2 = r$m
    r = lsch(r$mix, r$beta, r$m, mix2, beta2, m2, x, which=which,
             model=model, brkt=ifelse(alpha == 1, TRUE, FALSE))
    if( r$conv != 0 ) break
    if(any(r$mix$pr == 0)) {
      j = r$mix$pr == 0
      r$mix = dden(r$mix$pt[!j], r$mix$pr[!j])
      r = bfgs(r$mix, r$beta, r$m, x, tol, maxit, which=which, model=model)
      return(r)
    }
    if( r$ll >= old.r$ll && r$ll <= old.r$ll + tol ) {convergence = 0; break}
    prmt = c(if(which[1]) r$mix$pr[-(k1+1)] else NULL,
             if(which[2]) r$mix$pt else NULL,
             if(which[3]) r$beta else NULL,
             if(which[4]) r$m else NULL)
    d = prmt - old.prmt
    g = r$grad - old.r$grad
    dg = sum(d * g)
    if( dg < 0 ) D = D + (1 + drop(t(g) %*% D %*% g) / dg) * outer(d, d) / dg -
                     (outer(d, g) %*% D + D %*% outer(g, d)) / dg
  }
  r$num.iterations = i
  r
}

lsch = function(mix1, beta1, m1, mix2, beta2, m2, x, maxit=100,
                which=c(1,1,0,0), brkt=FALSE, model = c("normmix", "spsym")) {
  model = match.arg(model)
  k = length(mix1$pt)
  convergence = 1
  d1 = c(if(which[1]) mix2$pr[-k] - mix1$pr[-k] else NULL,
         if(which[2]) mix2$pt - mix1$pt else NULL,
         if(which[3]) beta2 - beta1 else NULL,
         if(which[4]) m2 - m1 else NULL)
  d1.norm = sqrt(sum(d1*d1))
  s = d1 / d1.norm
  dl1 = dll.snpmle(x, mix1, beta1, m1, which=c(1,which))
  ll1 = dl1$ll
  grad1 = c(if(which[1]) dl1$dp[-k] - dl1$dp[k] else NULL,
            if(which[2]) dl1$dt else NULL,
            if(which[3]) dl1$db else NULL,
            if(which[4]) dl1$dm else NULL)
  g1s = sum(grad1 * s)
  g1d1 = sum(grad1 * d1)
  if(d1.norm == 0 || g1s <= 0)
    return(list(mix=mix1, beta=beta1, m=m1, grad=grad1, ll=ll1, convergence=3))
  a = 0
  b = 1
  if(which[1] && any(mix2$pr == 0)) brkt = FALSE
  for(i in 1:maxit) {
    repeat {
      m0 = dden( (1-b) * mix1$pt + b * mix2$pt, (1-b) * mix1$pr + b * mix2$pr )
      beta = (1-b) * beta1 + b * beta2
      m = (1-b) * m1 + b * m2
      if( valid.snpmle(x, beta, m0, m) ) break
      brkt = FALSE
      b = a + 0.9 * (b - a)
      if(b <= .Machine$double.xmin) stop("valid failed")
    }
    dl = dll.snpmle(x, m0, beta, m, which=c(1,which))
    ll = dl$ll
    grad = c(if(which[1]) dl$dp[-k] - dl$dp[k] else NULL,
             if(which[2]) dl$dt else NULL,
             if(which[3]) dl$db else NULL,
             if(which[4]) dl$dm else NULL)
    gs = sum(grad * s)
    if( brkt && gs > g1s * .5 && ll >= ll1 + g1d1 * b * .33 ) {a = b; b = 2*b}
    else break
  }
  if(i == maxit) brkt = FALSE
  alpha = b
  for(i in 1:maxit) {
    g1d = g1d1 * alpha
    if( ll >= ll1 && ll + g1d <= ll ) {convergence=2; break}
    if( brkt ) {
      if( ll >= ll1 + g1d * .33 && abs(gs) <= g1s * .5 ) {convergence=0; break}
      if( ll >= ll1 + g1d * .33 && gs > g1s * .5 ) a = alpha else b = alpha
    }
    else {if( ll >= ll1 + g1d * .33 ) {convergence=0; break} else b = alpha}
    alpha = (a + b) * .5
    m0 = dden((1-alpha) * mix1$pt + alpha * mix2$pt,
              (1-alpha) * mix1$pr + alpha * mix2$pr)
    beta = (1-alpha) * beta1 + alpha * beta2
    m = (1-alpha) * m1 + alpha * m2
    dl = dll.snpmle(x, m0, beta, m, which=c(1,which))
    ll = dl$ll
    grad = c(if(which[1]) dl$dp[-k] - dl$dp[k] else NULL,
             if(which[2]) dl$dt else NULL,
             if(which[3]) dl$db else NULL,
             if(which[4]) dl$dm else NULL)
    gs = sum(grad * s)
  }
  mix = dden((1-alpha) * mix1$pt + alpha * mix2$pt,
             (1-alpha) * mix1$pr + alpha * mix2$pr)
  list(mix=mix, beta=(1-alpha) * beta1 + alpha * beta2,
       m = (1-alpha) * m1 + alpha * m2, grad=grad, ll=ll,
       convergence=convergence, num.iterations=i)
}

initial.snpmle = function(x, init=NULL, start=NULL) {
  if( is.null(init) || is.null(init$mix) )
    initial(x, init$beta, init$m, init$mix, start=start)
}

logLik.snpmle = function(x, beta, m, mix) {
  ld = sweep(logd(x, beta, m, mix$pt, which=c(1,rep(0,4)))$ld, 2, log(mix$pr), "+")
  ma = apply(ld, 1, max)
  pid = exp(ld - ma)
  pis = rowSums(pid)
  ll = sum( log(pis) + ma )
  attr(ll, "p") = pid / pis
  attr(ll, "ma") = ma
  ll
}

dll.snpmle = function(x, mix, beta, m, which=c(1,rep(0,4)), ind=FALSE) {
  r = list()
  dl = logd(x, beta, m, mix$pt, which=c(1,which[4:3],0,which[5]))
  lpt = dl$ld
  ma = apply(lpt, 1, max)
  dmix = drop(exp(lpt - ma) %*% mix$pr) + 1e-100
  dpt = pmin(exp(lpt - ma), 1e100)
  dp = dpt / dmix
  p = sweep(dp, 2, mix$pr, "*")
  if(which[1] == 1) {
    r$ll = log(drop(exp(lpt - ma) %*% mix$pr)) + ma
    if(!ind) r$ll = sum(r$ll)
  }
  if(sum(which[2:5]) == 0) return(r)
  if(which[2] == 1) {
    r$dp = dp
    if(!ind) r$dp = colSums(dp)
  }
  if(sum(which[3:5]) == 0) return(r)
  if(which[3] == 1) {
    r$dt = p * dl$dt1
    if(!ind) r$dt = colSums(r$dt)
  }
  if(sum(which[4:5]) == 0) return(r)
  if(which[4] == 1) {
    r$db = apply(sweep(dl$db1, 1:2, p, "*"), c(1,3), sum)
    if(!ind) r$db = colSums(r$db)
  }
  if(sum(which[5]) == 0) return(r)
  r$dm = apply(sweep(dl$dm, 1:2, p, "*"), c(1,3), sum)
  if(!ind) r$dm = colSums(r$dm)
  r
}

valid.snpmle = function(x, beta, mix, m) valid(x, beta, mix, m) && all(mix$pr >= 0)

collapse.snpmle = function(mix, beta, m, x) {
  mix = sort.dden(mix)
  j = mix$pr == 0
  if(any(j)) mix = dden(mix$pt[!j], mix$pr[!j])
  if(is.null(attr(mix, "ll")))
    attr(mix, "ll") = logLik.snpmle(x, beta, m, mix)
  if( any(mix$pr <= 1e-3) ) {
    j = mix$pr > 1e-3
    mixt = mix
    mixt$pt = mixt$pt[j]
    mixt$pr = mixt$pr[j] / sum(mix$pr[j])
    attr(mixt, "ll") = logLik.snpmle(x, beta, m, mixt)
    if(attr(mixt,"ll")[1] + 1e-8 >= attr(mix,"ll")[1]) mix = mixt
  }
  p = attr(attr(mix, "ll"), "p")
  pj = colSums(p)
  if( any(pj == 0) ) {
    j = pj != 0
    mix$pt = mix$pt[j]
    mix$pr = mix$pr[j] / sum(mix$pr[j])
    attr(mix, "ll") = logLik.snpmle(x, beta, m, mix)
  }
  repeat {
    if(length(mix$pt) == 1) break
    prec = min(diff(mix$pt))
    mixt = unique(mix, prec=c(prec,0))
    attr(mixt, "ll") = logLik.snpmle(x, beta, m, mixt)
    if(attr(mixt,"ll")[1] + 1e-8 >= attr(mix,"ll")[1]) mix = mixt else break
  }
  mix
}

dden = function(pt, pr=1) {
  if (is.null(pt)) d = list(pt=NULL, pr=NULL)
  else {
    k = max(length(pt), length(pr), na.rm=TRUE)
    pt = rep(pt, len=k)
    if(is.null(pr)) pr = rep(1, length(pt)) else pr = rep(pr, len=k)
    d = list(pt=pt, pr=pr/sum(pr))
  }
  class(d) = "dden"
  d
}

is.null.dden = function (d) is.null(d$pt)
is.dden = function (d) any(class(d) == "dden")

print.dden = function (d, ...) {
  if (is.null(d)) b = matrix(nrow=0, ncol=2)
  else b = cbind(d$pt, d$pr)
  dimnames(b) = list(NULL, c("pt", "pr"))
  print(b, ...)
}

sort.dden = function(d) {
  if( is.null(d) ) return(d)
  index = order(d$pt)
  d$pt = d$pt[index]
  d$pr = d$pr[index]
  d
}

is.unsorted.dden = function(d) is.unsorted(d$pt)

unique.dden = function(d, prec=0) {
  if( length(d$pt) == 1 ) return(d)
  if( is.unsorted.dden(d) ) d = sort.dden(d)
  prec = rep(prec, len=2)
  if ( all(prec < 0) ) return(d)
  pt2 = pt = d$pt
  pr2 = pr = d$pr
  j  = pr <= prec[2]
  pt = pt[!j]
  pr = pr[!j]
  index = 0
  repeat {
    if( length(pt) == 0 ) break
    j = abs(pt[1] - pt) <=  prec[1]
    index = index + 1
    pt2[index] = weighted.mean(pt[j], pr[j])
    pr2[index] = sum( pr[j] )
    pt = pt[!j]
    pr = pr[!j]
  }
  dden(pt=pt2[1:index], pr=pr2[1:index])
}



# =========
# Selection
# =========

# Information criteria

ic = function(x, from, to, by, which, model){
  mix = m = aic = bic = numpara = ll = NULL
  beta = rev(seq(from, to, by=by))
  betaveclen = length(beta)
  out = vector("list", betaveclen)
  for(i in 1:betaveclen){
    out[[i]]     = cnmms(x, init=list(mix=mix,beta=beta[i],m=m), which=which, model=model)
    mix          = out[[i]]$mix
    m            = out[[i]]$m
    numpara[i]   = length(out[[i]]$grad)
    ll[i]        = out[[i]]$ll
    aic[i]       = -2*ll[i] + 2 * numpara[i]
    bic[i]       = -2*ll[i] + log(length(x)) * numpara[i]
    if(model == "normmix") m = NULL
  }
  ic_beta = list(beta=beta, aic=aic, bic=bic, numpara=numpara, ll=ll)
  betaoptaic = ic_beta$beta[which.min(ic_beta$aic)]
  betaoptbic = ic_beta$beta[which.min(ic_beta$bic)]
  c(ic_beta, list(betaoptaic=betaoptaic,betaoptbic=betaoptbic))
}

# Cross-validation

dnormmix = function(x, para){
  out = rep(0,length(x))
  for (i in 1:length(para$mix$pt)) out = out + para$mix$pr[i]*dnorm(x,para$mix$pt[i],para$beta)
  out
}

dspsym = function(x, para){
  out = rep(0,length(x))
  for(i in 1:length(para$mix$pt)) out = out + 0.5*para$mix$pr[i]*dnorm(x,para$mix$pt[i],para$beta) + 0.5*para$mix$pr[i]*dnorm(x,2*para$m-para$mix$pt[i],para$beta)
  out
}

int_pdfsq = function(para, model) {
  if(model=="normmix") {
    r = outer(x<-para$mix$pr,x,"*")
    s = outer(x<-para$mix$pt,x,"-")
  }
  if(model=="spsym") {
    r = outer(x<-c(para$mix$pr/2,para$mix$pr/2),x,"*")
    s = outer(x<-c(para$mix$pt,2*para$m-para$mix$pt),x,"-")
  }
  index = col(r) < row(r)
  sum(c(diag(r),2*r[index])*dnorm(c(diag(s),s[index]),0,sqrt(2)*para$beta))
}

cv = function(x, K, from, to, by, seed, R, which, model) {
  n = length(x)
  hvec = rev(seq(from, to, by=by))
  hveclen = length(hvec)
  cvmat = cvklmat = matrix(0, nr=R, nc=hveclen)
  for(r in 1:R) {
    firstTerm = secondTerm = klTerm = matrix(NA, nr=K, nc=hveclen)
    set.seed(seed * 100 + r)
    allfolds = split(sample(1:n), rep(1:K,length=n))
    for(ii in seq(K)) {
      omit = allfolds[[ii]]
      xx = x[-omit]
      mix = m = NULL
      for(i in 1:length(hvec)) {
        fitleave = cnmms(xx, init=list(mix=mix,beta=hvec[i],m=m), which=which, model=model)
        mix = fitleave$mix
        m = fitleave$m
        firstTerm[ii,i] = int_pdfsq(fitleave, model)
        if(model=="normmix") {
          ll = log(dnormmix(x[omit], fitleave))
          secondTerm[ii,i] = mean(exp(ll))
          klTerm[ii,i] = mean(ll)
        }
        if(model=="spsym") {
          ll = log(dspsym(x[omit], fitleave))
          secondTerm[ii,i] = mean(exp(ll))
          klTerm[ii,i] = mean(ll)
        }
      }
    }
    cvmat[r,] = (colSums(firstTerm) - 2*colSums(secondTerm))/K
    cvklmat[r,] = - colSums(klTerm)/K
  }
  cvscore = list(hvec=hvec, cv=colMeans(cvmat), cvkl=colMeans(cvklmat))
  hoptcv  = cvscore$hvec[which.min(cvscore$cv)]
  hoptcvkl = cvscore$hvec[which.min(cvscore$cvkl)]
  c(cvscore, list(hoptcv=hoptcv,hoptcvkl=hoptcvkl))
}

# =====
# Model
# =====

# Class MDE

print.normmix = function(x, ...) print(as.numeric(x), ...)
length.normmix = function(x) length(as.numeric(x))
range.normmix = function(x, beta, ...) range(as.numeric(x), ...)

initial.normmix = function(x, beta=NULL, m=NULL, mix=NULL, start=1) {
  x = as.numeric(x)
  n = length(x)
  if(is.null(beta)) beta = sd(x)
  if(length(beta) != 1 || beta <= 0) stop("initial beta incorrect")
  if(is.null(mix) || is.null(mix$pt)) {
    if(start == 1) {
      k = max(ceiling( (max(x) - min(x)) / (5*beta) ), 10)
      mu = seq(min(x), max(x), len=k)
      counts = hist(x, mu, plot=FALSE)$counts
      mix = dden(c(mu[1], mu[-1][counts != 0]))
    }
    else if(start == 2) mix = dden(unique(quantile(x, p=seq(0,1,len=10), type=1)))
    else mix = dden(x)
  }
  list(beta=beta, m=m, mix=mix)
}

valid.normmix = function(x, beta, mix, m) beta > 0

logd.normmix = function(x, beta, m, pt, which=c(1,0,0,0,0)) {
  x = as.numeric(x)
  dl = vector("list", 4)
  names(dl) = c("ld","db1","dt1","dt2")
  if(which[1] == 1) dl$ld = - 1/2*log(2*pi*beta^2)-outer(x, pt, "-")^2/beta^2/2
  if(which[2] == 1) dl$db1 = array(- 1/beta + outer(x, pt, "-")^2/beta^3,
                                   dim=c(length(x), length(pt), 1))
  if(which[3] == 1) dl$dt1 = outer(x, pt, "-")/beta^2
  if(which[4] == 1) dl$dt2 = matrix(- 1/beta^2, nrow=length(x), ncol=length(pt))
  dl
}

# Class SMDE 

print.spsym = function(x, ...) print(as.numeric(x), ...)
length.spsym = function(x) length(as.numeric(x))
range.spsym = function(x, beta, ...) range(as.numeric(x), ...)

initial.spsym = function(x, beta=NULL, m=NULL, mix=NULL, start=1) {
  x = as.numeric(x)
  n = length(x)
  if(is.null(beta)) beta = sd(x)
  if(length(beta) != 1 || beta <= 0) stop("initial beta incorrect")
  if(start == 1) {
      if(is.null(mix) || is.null(mix$pt)) mix = dden(unique(quantile(x, p=seq(0,1,len=10), type=1)))
      if(is.null(m)) m = median(x)
  }
  else {
      if(is.null(mix) || is.null(mix$pt)) mix = dden(x)
      if(is.null(m)) m = median(x)
  }
  list(beta=beta, m=m, mix=mix)
}

valid.spsym = function(x, beta, mix, m) beta > 0 && (m >= min(x) & m <= max(x))

logd.spsym = function(x, beta, m, pt, which=c(1,0,0,0,0)) {
  x = as.numeric(x)
  dl = vector("list", 5)
  names(dl) = c("ld","db1","dt1","dt2","dm")
  outerf1 = outer(x, pt, "-")
  outerf2 = outer(x, 2*m - pt, "-")
  f1 = sweep(dnorm(sweep(outerf1, 2, beta, "/")), 2, beta, "/")
  f2 = sweep(dnorm(sweep(outerf2, 2, beta, "/")), 2, beta, "/")
  dcomp = 0.5*(f1 + f2)
  dcomp[dcomp<1e-100] = 1e-100
  if(which[1] == 1) dl$ld = log(dcomp)
  if(which[2] == 1) dl$db1 = array((f1*(outerf1^2 - beta^2)/beta^3  +
                                    f2*(outerf2^2 - beta^2)/beta^3)/(2*dcomp),
                                    dim=c(length(x), length(pt), 1))
  if(which[3] == 1 || which[4] == 1) {
    dl$dt1 = (f1*outerf1/beta^2 - f2*outerf2/beta^2)/(2*dcomp)
    if(which[4] == 1)
      dl$dt2 = matrix(((f1*(outerf1^2 - beta^2)/beta^4  +
                        f2*(outerf2^2 - beta^2)/beta^4)/(2*dcomp)) - (dl$dt1)^2,
                        nrow=length(x), ncol=length(pt))
  }
  if(which[5] == 1) dl$dm = array((f2*outerf2/beta^2)/dcomp, dim=c(length(x), length(pt), 1))
  dl
}
